/*
 *******************************************************************
 *              TMark favorites plugin for TVBrowser               *
 *                                                                 *
 * Copyright (C) 2006 Tomas Schackert.                             *
 * Contact koumori@web.de                                          *
 *******************************************************************

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, in version 2 of the License.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program, in a file called LICENSE in the top
 directory of the distribution; if not, write to 
 the Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA  02111-1307  USA
 
 *******************************************************************/

package tmarkplugin.properties;



/**
 * tmarkplugin.config.Property for a conditional property
 * @author Tomas Schackert
 *
 */
public class ConditionalProperty extends Property{
	private String condkey;
	private boolean cond=false;
	private boolean defaultcond;
	private boolean loaded=false;
	
	ConditionalProperty(PropertySource props){
		super(props);
		this.condkey=null;
		this.defaultcond=true;
		this.cond=true;
	}
	ConditionalProperty(PropertySource props,String condkey,boolean defaultcond){
		super(props);
		this.condkey=condkey;
		this.defaultcond=defaultcond;
	}
	/**
	 * 
	 */
	private void load() {
		if (this.condkey!=null)cond=this.getBoolProperty(condkey,defaultcond);
		else cond=true;
		this.loaded=true;
		
	}
	public void setCondition(boolean cond) {
		setCondition(cond,true); 
	}
	public void setCondition(boolean cond,boolean fireNofiy) {
		if (this.cond==cond)return;
		this.cond=cond;
		if (this.condkey!=null)this.setBoolProperty(condkey,this.cond=cond);
		loaded=true;
		if (fireNofiy)this.fireChanged();
	}
	
	public boolean condition(){
		if (!loaded) load();
		return cond;
	}
	public boolean useCondition(){
		return this.condkey!=null;
	}
	public void setCondition(ConditionalProperty src) {
		setCondition(src,true); 
	}
	/**
	 * @param src
	 */
	public void setCondition(ConditionalProperty src,boolean fireNofiy) {
		if (!src.loaded)src.load();
		this.condkey=src.condkey;
		this.defaultcond=src.defaultcond;
		if (this.condkey!=null)this.setBoolProperty(condkey,this.cond=src.cond);
		loaded=true;
		if (fireNofiy)this.fireChanged();
	}
	public void reset() {
		setCondition(this.defaultcond); 
	}
}
