/*
 *******************************************************************
 *              TMark favorites plugin for TVBrowser               *
 *                                                                 *
 * Copyright (C) 2006 Tomas Schackert.                             *
 * Contact koumori@web.de                                          *
 *******************************************************************

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, in version 2 of the License.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program, in a file called LICENSE in the top
 directory of the distribution; if not, write to 
 the Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA  02111-1307  USA
 
 *******************************************************************/

package tmarkplugin.properties;

import java.awt.Color;
import java.awt.Font;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.Vector;

/**
 * @author Tomas
 *
 */
abstract public class PropertySource {
	//default implementation for comverting properties from/to String
	static public int convStringToInt(String val,int defaultValue){
		try{
			if (val==null || val.length()==0) return defaultValue; 
			int result=Integer.parseInt(val);
			return result;
		}catch (NumberFormatException nfe){
			return defaultValue;
		}
	}
	static public String convIntToString(int value){
		return Integer.toString(value);
	}
	static public int[] convStringToIntArray(String val,int defaultValue[]){
		if (val==null || val.length()==0) return defaultValue;
		val=val.trim().substring(1,val.length()-1);	//remove []
		Vector tmp=new Vector();
		String array[];
		int i;
		try{
	
			array=val.split(",");
			for (i=0;i<array.length;i++){
				array[i]=array[i].trim();
				if (array[i].length()<=0) continue;
				tmp.add(new Integer(array[i]));
			}
	
			int result[]=new int[tmp.size()];
			for (i=0;i<result.length;i++){
				result[i]=((Integer)tmp.get(i)).intValue();
			}
			return result;
		}catch (NumberFormatException nfe){
			return defaultValue;
		}
	}
	static public String convIntArrayToString(int value[]){
		return Arrays.toString(value);
	}
	static public String mask(String s){
		s=s.replace("\n","\\n");
		return s;
	}
	static public String unmask(String s){
		s=s.replace("\\n","\n");
		return s;
	}
	static public String convStringArrayToString(String value[]){
		if (value.length==0) return null;
		StringBuffer sb=new StringBuffer();
		for (int i=0;i<value.length-1;i++){
			sb.append(mask(value[i]));
			sb.append("\n");
		}
		sb.append(mask(value[value.length-1]));
		return sb.toString();
	}
	static public String[] convStringToStringArray(String value,String defaultValue[]){
		if (value==null) return defaultValue;
		String array[]=value.split("\n");
		for (int i=0;i<array.length;i++){
			array[i]=unmask(array[i]);
		}
		return array;
	}
	
	
	static public String convBoolToString(boolean value) {
		return value?"true":"false";
	}
	static public boolean convStringToBool(String value,boolean defaultval){
		boolean result=defaultval;
		if (value!=null)result=value.trim().equalsIgnoreCase("true");
		return result;
	}
	static public String convFloatToString(float value) {
		return Float.toString(value); 
	}
	static public float convStringToFloat(String value,float defaultval){
		float result=defaultval;
		if (value!=null){
			try{
				result=Float.parseFloat(value);
			}catch (NumberFormatException nfe){
			}
		}
		return result;
	}

	static public String convLongToString(long value) {
		return Long.toString(value);
	}
	static public long convStringToLong(String value,long defaultval){
		long result=defaultval;
		if (value!=null){
			try{
				result=Long.parseLong(value);
			}catch (NumberFormatException nfe){
			}
		}
		return result;
	}
	static public Color convStringToColor(String val, Color defaultValue) {
		//default implementation for storing color properties as Strings
		if (val==null || val.length()==0) return defaultValue;
		val=val.trim().substring(1,val.length()-1);//remove ()
		Vector tmp=new Vector();
		String array[];
		int i;
		try{

			array=val.split(",");
			for (i=0;i<array.length;i++){
				array[i]=array[i].trim();
				if (array[i].length()<=0) continue;
				tmp.add(new Integer(array[i]));
			}
			int r=((Integer)tmp.get(0)).intValue();
			int g=((Integer)tmp.get(1)).intValue();
			int b=((Integer)tmp.get(2)).intValue();
			int a=(tmp.size()>3)?((Integer)tmp.get(3)).intValue():255;
			return new Color(r, g, b, a);
		}catch (NumberFormatException nfe){
			return defaultValue;
		} 
	}

	static public String convColorToString(Color color){
		//default implementation for storing color properties as Strings
		if (color==null){
			return null;
		}else{
			return "("+color.getRed()+","+color.getGreen()+","+color.getBlue()+","+color.getAlpha()+")";
		}	
	}


	
	//default implementations for some property types, stores data as Strings
	public int getIntProperty(String key,int defaultValue){
		return convStringToInt(getProperty(key,null),defaultValue);
	}
	public void setIntProperty(String key,int value) {
		setProperty(key,convIntToString(value));
	}
	public void setIntArrayProperty(String key,int value[]){
		setProperty(key,convIntArrayToString(value));
	}
	public int[] getIntArrayProperty(String key,int defaultValue[]){
		return convStringToIntArray(getProperty(key,null),defaultValue);
	}
	
	public void setStringArrayProperty(String key,String value[]){
		setProperty(key,convStringArrayToString(value));
	}
	public String[] getStringArrayProperty(String key,String defaultValue[]){
		return convStringToStringArray(getProperty(key,null),defaultValue);
	}
	public void setBoolProperty(String key,boolean value) {
		setProperty(key,convBoolToString(value));
	}
	public boolean getBoolProperty(String key,boolean defaultval){
		return convStringToBool(getProperty(key,null),defaultval);
	}
	public void setFloatProperty(String key,float value) {
		setProperty(key,convFloatToString(value)); 
	}
	public float getFloatProperty(String key,float defaultval){
		return convStringToFloat((String)getProperty(key,null),defaultval);
	}
	public void setLongProperty(String key,long value) {
		setProperty(key,convLongToString(value));
	}
	public long getLongProperty(String key,long defaultval){
		return convStringToLong((String)getProperty(key,null),defaultval);
	}
	abstract public String setProperty(String key, String value);
	abstract public String getProperty(String key, String defaultvalue);
	abstract public Enumeration getPropertyKeys();
	
	public Color getColorProperty(String key, Color defaultValue) {
		return convStringToColor(getProperty(key,null),defaultValue);
	}

	public void setColorProperty(String key, Color color){
		setProperty(key,convColorToString(color));
	}
	abstract public Font getFontProperty(String basekey, Font defaultvalue);
	abstract public void setFontProperty(String basekey, Font value) ;
	


}
